@@ -1,5 +1,5 @@
 ;;; jde.el -- Integrated Development Environment for Java.
-;; $Revision: 1.58 $ $Date: 1998/06/30 03:35:10 $ 
+;; $Revision: 1.76 $ $Date: 1998/12/09 00:56:31 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -22,10 +22,6 @@
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
-;;;###autoload
-(defconst jde-version "2.0.5"
-  "JDE version number.")
-
 ;;; Commentary:
 
 ;; This is one of a set of packages that make up the 
@@ -38,9 +34,14 @@
 ;; Please send any comments, bugs, or upgrade requests to
 ;; Paul Kinnucan at paulk@mathworks.com.
 
-
 ;;; Code:
 
+
+;;;###autoload
+(defconst jde-version "2.1.4"
+  "JDE version number.")
+
+
 (defvar jde-xemacsp (string-match "XEmacs" emacs-version)
   "Non-nil if we are running in the XEmacs environment.")
 
@@ -51,6 +52,8 @@
 (require 'font-lock)
 (require 'cc-mode)
 (c-initialize-cc-mode)
+(require 'cus-edit)
+(require 'jde-compile)
 (require 'jde-db)
 (require 'jde-run)
 (require 'jde-make)
@@ -59,6 +62,8 @@
 (require 'imenu)
 (require 'speedbar)
 (require 'browse-url)
+(require 'beanshell)
+(require 'jde-wiz)
 
 ;; This is copied straight out of andersl-java-font-lock.el
 ;; Necessary to set here because andersl assumes that the
@@ -112,6 +117,34 @@
   :group 'jde
   :prefix "jde-")
 
+(defcustom jde-key-bindings
+  (list (cons "\C-c\C-v\C-c" 'jde-compile)
+	(cons "\C-c\C-v\C-r" 'jde-run)
+	(cons "\C-c\C-v\C-d" 'jde-db)
+	(cons "\C-c\C-v\C-b" 'jde-build)
+	(cons "\C-c\C-v\C-a" 'jde-run-menu-run-applet)
+	(cons "\C-c\C-v\C-n" 'jde-browse-jdk-doc)
+	(cons "\C-c\C-v\C-p" 'jde-save-project)
+	(cons "\C-c\C-v\C-l" 'jde-gen-println))
+  "*Specifies key bindings for the JDE.
+The value of this variable is an association list. The car of
+each element specifies a key sequence. The cdr specifies 
+an interactive command that the key sequence executes. To enter
+a key with a modifier, type C-q followed by the desired modified
+keystroke. For example, to enter C-s (Control s) as the key to be
+bound, type C-q C-s in the key field in the customization buffer."
+  :group 'jde-project
+  :type '(repeat
+	  (cons :tag "Key binding"
+	   (string :tag "Key")
+	   (function :tag "Command")))
+  :set '(lambda (sym val)
+	  (if (boundp 'jde-mode-map)
+	      (mapc (lambda (binding)
+		      (define-key jde-mode-map (car binding) (cdr binding)))
+		    val))
+	  (set-default sym val)))
+
 (defcustom jde-jdk-doc-url "http://www.javasoft.com/products/jdk/1.1/docs/index.html"
   "*URL of JDK documentation. 
 This can point to a remote or local copy of the documentation. By
@@ -141,10 +174,8 @@ The semicolons in the classpath confuse the shell."
   :group 'jde-project
   :type 'boolean)
 
-(defcustom jde-project-name ""
-"*Specifies name of project to which the current buffer belongs."
-  :group 'jde-project
-  :type 'string)
+(defvar jde-project-name "default"
+"Specifies name of project to which the current buffer belongs.")
 
 (defcustom jde-project-file-name "prj.el"
   "*Specify name of JDE project file.
@@ -188,175 +219,11 @@ entered in the minibuffer."
 "History of compiler arguments entered in the minibuffer.")
 
 
-(defcustom jde-entering-java-buffer-hooks nil
-"*Lists functions to run when entering a Java source buffer"
-  :group 'jde-project
-  :type 'hook)
-
 (defcustom jde-build-use-make nil
 "*If true, use make to build JDE projects."
   :group 'jde-project
   :type 'boolean)
 
-
-(defgroup jde-compile-options nil
-  "JDE Compiler Options"
-  :group 'jde
-  :prefix "jde-compile-option-")
-
-(defcustom jde-compile-option-command-line-args ""
-  "*Specify options as a string of command-line arguments.
-The value of this variable should be a string of switches understood
-by the compiler, for example, -depend -g. This variable is intended to
-be used to set compile options not otherwise defined by the JDE, in
-particular, options not defined by javac but used by another compiler
-that you might want to use with the JDE."
-  :group 'jde-compile-options
-  :type 'string)
-
-(defcustom jde-compile-option-classpath nil
-"*Specify paths of classes required to compile this project.
-The JDE uses the specified paths to construct a -classpath
-argument to pass to the compiler. This option overrides the
-`jde-global-classpath' option."
-  :group 'jde-compile-options
-  :type '(repeat (string :tag "Path")))
-
-
-(defcustom jde-compile-option-directory ""
-  "*Specifies the root directory of the class file hierarchy.
-The compiler places compiled classes in the specified
-directory. For example, specifying the class
-directory as: 
-  
-  C:\\users\\dac\\classes
-
-causes the class files for the classes in the MyProgram.java source
-file to be saved in the directory C:\\users\\dac\\classes. If your class 
-is in the package demos\\awt, the class files would be placed in directory
-C:\\users\\dac\\classes\\demos\\awt."
-  :group 'jde-compile-options
-  :type 'string)
-
-(defcustom jde-compile-option-deprecation nil
-  "*Warn use or override of a deprecated member or class. 
-A member or class is deprecated if its documentation comment contains
-the @deprecated tag. The compiler will emit a warning at the end of
-compilation whether or not the deprecation option is on; this option
-causes the location of each individual use or override to be noted.
-
-Deprecated members or classes are deliberately not mentioned if the
-source file containing the deprecation is being recompiled.  This can
-happen because the file is on the command line or because the depend
-option is on and the source file is out of date.
-"
-  :group 'jde-compile-options
-  :type 'boolean)
-
-
-(defcustom jde-compile-option-debug nil
-"*Generate information about local variables for debug tools.
-By default, only line number information is generated.
-
-Prior to JDK 1.2, the the debug and optimize options were
-mutually exclusive. In JDK 1.2, it is possible to combine debug and
-optimize, but the shortcuts taken by optimized code may occasionally
-produce surprising debugging results. For example, declared variables
-may not exist and code may appear to move or not be executed at all."
-  :group 'jde-compile-options
-  :type 'boolean)
-
-(defcustom jde-compile-option-nodebug nil
-"*Do not generate line number or local variable debugging information."
-  :group 'jde-compile-options
-  :type 'boolean)
-
-(defcustom jde-compile-option-optimize nil
-"*Directs the compiler to try to generate faster code. 
-This may slow down compilation, make larger class files, and/or make
-it difficult to debug.
-
-Prior to 1.2, the optimize option tried to inline methods across
-classes. This created compatibility problems and sometimes generated
-illegal bytecode. The optimize option also implicitly turned on the
-depend option and implicitly turned off the debug option.
-
-In JDK 1.2, the optimize option no longer inlines across classes and
-so may safely be used for any java compilation. Optimize no longer
-implicitly turns on depend or implicitly turns off debug."
-  :group 'jde-compile-options
-  :type 'boolean)
-
-(defcustom jde-compile-option-optimize-interclass nil
-"*Enable interclass optimizations.
-This option informs the compiler that all generated class files are
-guaranteed to be delivered and upgraded as a unit, enabling interclass
-optimizations that may otherwise break binary compatibility. Use this
-option with discretion.
-
-The Java Language Specification section 13.4.21 describes situations
-in which it is legal to use a java compiler to inline methods. The
-compiler will only optimize code for which source is available during
-the compilation, so the only .java files discoverable by the compiler
-should be for classes intended to be delivered or upgraded as a
-unit. In particular, ensure that no sources for other classes are
-accessible on CLASSPATH, keeping in mind that the present working
-directory, `.', is appended to CLASSPATH.
-
-To ensure that a product is able to run on 1.2 as well as future
-binary-compatible java virtual machines, one must ensure that any
-sources for JDK 1.2 classes are never available along CLASSPATH while
-using the interclass optimization option.
-
-Line number debugging information in class files compiled with the
-interclass optimization option may refer to lines of a different
-source file. This is a limitation of the class file format, which does
-not allow the originating file to be encoded along with the line
-number.
-
-This option implicitly turns on the dependency option."
-  :group 'jde-compile-options
-  :type 'boolean)
-
-(defcustom jde-compile-option-depend nil
-"*Analyze dependencies.
-Causes recompilation of class files on which the source files given as
-command line arguments recursively depend. Without this option, only
-files that are directly depended on and missing or out-of-date will be
-recompiled. Recompilation does not extend to missing or out-of-date
-files only depended on by already up-to-date class files."
-  :group 'jde-compile-options
-  :type 'boolean)
-
-(defcustom jde-compile-option-vm-args nil
-"*Specify command-line arguments for Java interpreter.
-Passes the specified arguments to the Java interpreter that runs the
-compiler. The argument should not contain spaces. This is useful for
-adjusting the compiler's execution environment or memory usage."
-  :group 'jde-compile-options
-  :type '(repeat (string :tag "Option")))
-
-(defcustom jde-compile-option-verbose nil
-"*Print verbose messages.
-Causes the compiler and linker to print out messages about what source
-files are being compiled and what class files are being loaded."
-  :group 'jde-compile-options
-  :type 'boolean)
-
-(defcustom jde-compile-option-nowarn nil
-"*Turn off warnings.
-If this option is specified, the compiler does not print out any
-warnings."
-  :group 'jde-compile-options
-  :type 'boolean)
-
-(defcustom jde-compile-option-encoding nil
-"*Specify the source file encoding name, such as EUCJIS\\SJIS.
-If this option is not specified, then the platform default converter
-is used."
-  :group 'jde-compile-options
-  :type 'boolean)
-
 (defcustom jde-enable-abbrev-mode nil
 "*Enable expansion of abbreviations in jde-mode.
 See `jde-mode-abbreviations' for more information."
@@ -438,6 +305,11 @@ current buffer. Default is javac."
        (setq jde-compiler "javac")
      (setq jde-compiler compiler)))
 
+(defvar jde-classpath-separator (if (eq system-type 'cygwin32) 
+				    ";" path-separator)
+  "The separator to use in a classpath.
+This is usually the same as `path-separator'")
+  
 (defun jde-path-string-to-list (paths)
  "Converts a string of paths to a list of paths.
 It is assumed that the default path separator for the
@@ -445,14 +317,14 @@ current platform (e.g., semicolon on Win32) separates
 the paths."
  (let ((path-list (list))
        (m 0)
-       (n (string-match path-separator paths)))
+       (n (string-match jde-classpath-separator paths)))
    (while n
      (let ((path (substring paths m n)))
        (if path
 	   (setq path-list
 		 (cons path path-list)))
        (setq m (+ n 1))
-       (setq n (string-match path-separator paths m))))
+       (setq n (string-match jde-classpath-separator paths m))))
    (setq n (length paths))
    (if (and (> n 0) (< m n))
        (let ((path (substring paths m n)))
@@ -469,20 +341,6 @@ interpreter."
    "sEnter classpath: ")
   (setq jde-global-classpath (jde-path-string-to-list classpath)))
 
-(defun jde-build-classpath-arg (path-list quote)
-"Build a classpath from a list of paths."
-  (let ((classpath "")
-	(len (length path-list))
-	(n 0))
-    (while (< n len)
-      (setq classpath (concat classpath
-			      (if (> n 0)
-				  path-separator)
-			      (elt path-list n)))
-      (setq n (1+ n)))
-    (if quote
-	(setq classpath (concat "\"" classpath "\"")))
-    (setq classpath (concat "-classpath " classpath))))
 
 (defun jde-build-compile-vm-args ()
   (let ((args " ")
@@ -510,70 +368,6 @@ stored in the variable jde-jdk-doc-url to locate the JDK documentation."
 	  " "
 	  (file-name-nondirectory buffer-file-name)))
 
-(defun jde-get-compile-options ()
-"Constructs a command-line argument string for compiler.
-The string consists of the contents of the jde-compile-options
-variable concatenated with the various jde-compile-option
-settings."
-  (let (options)
-
-    (if jde-compile-option-classpath
-	(setq options 
-	      (jde-build-classpath-arg
-	       jde-compile-option-classpath jde-quote-classpath))
-      (if jde-global-classpath
-	  (setq options
-		(jde-build-classpath-arg
-		 jde-global-classpath jde-quote-classpath))))
-	     
-    (if jde-compile-option-debug
-	(setq options (concat options " -g")))
-
-    (if (not (string= jde-compile-option-directory ""))
-	(setq options
-	      (concat options 
-		" -d "
-		jde-compile-option-directory)))
-
-    (if jde-compile-option-deprecation
-	(setq options (concat options " -deprecation")))
-
-    (if jde-compile-option-nodebug
-	(setq options (concat options " -g:nodebug")))
-
-    (if jde-compile-option-optimize
-	(setq options (concat options " -O")))
-
-    (if jde-compile-option-optimize-interclass
-	(setq options (concat options " O:interclass")))
-
-    (if jde-compile-option-depend
-	(setq options (concat options " -depend")))
-
-    (if jde-compile-option-vm-args
-	(setq options 
-	      (concat options (jde-build-compile-vm-args))))
-
-    (if jde-compile-option-verbose
-	(setq options (concat options " -verbose")))
-
-    (if jde-compile-option-nowarn
-	(setq options (concat options " -nowarn")))
-
-    (if (not (string= jde-compile-option-command-line-args ""))
-	(setq options (concat options " " 
-			      jde-compile-option-command-line-args)))
-
-    options))
-
-;;;###autoload
-(defun jde-set-compile-options (options)
-  "Sets the compile options.
-Enter the options as you would on the command line, e.g.,
--depend -verbose."
-  (interactive
-   "sEnter options: ")
-  (setq jde-compile-option-command-line-args options))
 
 (defun jde-show-compile-options ()
   "Show the JDE Compile Options panel."
@@ -600,48 +394,6 @@ Enter the options as you would on the command line, e.g.,
   (interactive)
   (customize-apropos "jde-gen" 'groups))
 
-;;;###autoload
-(defun jde-compile ()
-  "Compile the Java program in the current buffer.
-This command invokes the compiler specified by `jde-compiler'
-with the options specified by the JDE customization variables
-that begin with `jde-compile'. If the variable
-`jde-read-compile-args' is non-nil, this command reads
-additional compilation options from the minibuffer, with
-history enabled."
-  (interactive)
-
-  (if jde-read-compile-args
-      (setq jde-interactive-compile-args
-	      (read-from-minibuffer 
-	       "Compile args: "
-	       jde-interactive-compile-args
-	       nil nil
-	       '(jde-interactive-compile-arg-history . 1))))
-
-  (let ((compile-command
-	 (jde-make-compile-command 
-	  jde-interactive-compile-args)))
-	  
-
-  
-
-    ;; Force save-some-buffers to use the minibuffer
-    ;; to query user about whether to save modified buffers.
-    ;; Otherwise, when user invokes jde-compile from
-    ;; menu, save-some-buffers tries to popup a menu
-    ;; which seems not to be supported--at least on
-    ;; the PC.
-    (if (eq system-type 'windows-nt)
-	(let ((temp last-nonmenu-event))
-	  ;; The next line makes emacs think that jde-compile
-	  ;; was invoked from the minibuffer, even when it
-	  ;; is actually invoked from the menu-bar.
-	  (setq last-nonmenu-event t)
-	  (save-some-buffers (not compilation-ask-about-save) nil)
-	  (setq last-nonmenu-event temp))
-      (save-some-buffers (not compilation-ask-about-save) nil))
-    (compile-internal compile-command "No more errors")))
 
 ;;;###autoload
 (defun jde-java-build ()
@@ -673,7 +425,7 @@ specified by `jde-run-application-class'."
 	  (find-file file-path)
 	  (setq jde-compile-option-depend t)
 	  (jde-compile)
-	  (setq jde-compile-option save-depend)))
+	  (setq jde-compile-option-depend save-depend)))
        (t
 	(message (concat "Could not find source for "
 			 jde-run-application-class))))))))
@@ -754,7 +506,17 @@ to run the application's main class."
 (defun jde-setup-syntax-coloring() 
   ;; Set up syntax coloring.
   (cond (window-system
-;	 (turn-on-font-lock)
+
+	 ;; If not XEmacs 20.1 turn on font lock.
+	 ;; (XEmacs 21 has font-lock on by default.)
+	 (if (or
+	      (not jde-xemacsp)
+	      (not
+	       (and
+		(eq emacs-major-version 21)
+		(eq emacs-minor-version 0))))
+	     (turn-on-font-lock))
+
 	 (setq font-lock-maximum-decoration t)
 
 	 (if (not jde-xemacsp)
@@ -784,9 +546,11 @@ to run the application's main class."
 	["Debug App"         jde-db t]
 	["Run Applet"        jde-run-menu-run-applet t]
 	["Build"             jde-build t]
+	["Interpret"         bsh t]
         ["-"                 ignore nil]
-	(list "Generate"
+	(list "Templates"
 	      ["Get/Set Pair..."  jde-gen-get-set t]
+	      ["Println..."       jde-gen-println t]
 	      (list "Listener"
 		    ["Action"          jde-gen-action-listener t]
 		    ["Window"          jde-gen-window-listener t]
@@ -794,6 +558,10 @@ to run the application's main class."
 		    )
 	      ["Other..."        jde-gen-code t]
 	      )
+	(list "Wizards"
+	      ["Override Method"     jde-wiz-override-method t]
+	      ["Implement Interface" jde-wiz-implement-interface t]
+	      )
 	["Speedbar"          speedbar-frame-mode t]
 	["Browse JDK Doc"    jde-browse-jdk-doc t]
 	(list "Options"
@@ -861,19 +629,6 @@ to run the application's main class."
   (unless (featurep 'infodock)
     (add-submenu '("File") jde-new-buffer-menu "Insert File...")))
 
-
-
-;; Define jde-mode accelerator keys
-(define-key jde-mode-map "\C-c\C-v\C-c" 'jde-compile)
-(define-key jde-mode-map "\C-c\C-v\C-r" 'jde-run)
-(define-key jde-mode-map "\C-c\C-v\C-d" 'jde-db)
-(define-key jde-mode-map "\C-c\C-v\C-b" 'jde-build)
-(define-key jde-mode-map "\C-c\C-v\C-s" 'speedbar-frame-mode)
-(define-key jde-mode-map "\C-c\C-v\C-a" 'jde-run-menu-run-applet)
-(define-key jde-mode-map "\C-c\C-v\C-n" 'jde-browse-jdk-doc)
-(define-key jde-mode-map "\C-c\C-v\C-u" 'jde-update-buffer)
-(define-key jde-mode-map "\C-c\C-v\C-p" 'jde-save-project)
-
 ;; Project File Functions
 
 (defun jde-root-dir-p (dir)
@@ -901,7 +656,8 @@ directory. Searches for the project file first in the source directory,
 then in ascendant directories. Uses the first file that it encounters."
   (let ((prj-file (jde-find-project-file default-directory)))
     (if prj-file
-	(load-file prj-file))))
+	(load-file prj-file)
+      (jde-set-variables-init-value))))
 
 ;;;###autoload
 (defun jde-open-project-file ()
@@ -939,38 +695,38 @@ Leave point at the location of the call, or after the last expression."
     (unless (bolp)
       (princ "\n"))))
 
+
 (defun jde-save-variables ()
   "Save all JDE variables in project file."
+  (jde-save-delete 'jde-set-project-name)
   (jde-save-delete 'jde-set-variables)
   (let ((standard-output (get-buffer jde-project-file-name)))
     (unless (bolp)
       (princ "\n"))
+
+    (princ "(jde-set-project-name ")
+    (prin1 jde-project-name)
+    (princ ")\n")
+
     (princ "(jde-set-variables ")
     (mapatoms
      (lambda (symbol)
        (when 
 	   (and (string-match "jde-" (symbol-name symbol))
 		(get symbol 'custom-type))
-	 (let ((value (symbol-value symbol))
-	       (requests (get symbol 'custom-requests))
-	       (now (not (or (get symbol 'standard-value)
-			     (and (not (boundp symbol))
-				  (not (get symbol 'force-value)))))))
-	   (when value
+	 (let ((value (symbol-value symbol)))	   
 	     (princ "\n '(")
 	     (princ symbol)
 	     (princ " ")
-	     (prin1 value)
-	     (cond (requests
-		    (if now
-			(princ " t ")
-		      (princ " nil "))
-		    (prin1 requests)
-		    (princ ")"))
-		   (now
-		    (princ " t)"))
-		   (t
-		    (princ ")"))))))))
+	     (prin1 (custom-quote value))
+	     ;; Check whether the user has changed the value of this
+	     ;; variable in a customization buffer. If so, save flag
+	     ;; so that custom knows that this value differs from
+             ;; standard value.
+	     (if (get symbol 'customized-value)
+		 (princ " t)")
+	       (princ ")"))		 
+	     ))))
       (princ ")")
       (save-excursion
 	(set-buffer (get-buffer jde-project-file-name))
@@ -978,6 +734,9 @@ Leave point at the location of the call, or after the last expression."
 	  (princ "\n"))
 	(save-buffer))))
 
+(defun jde-set-project-name (name)
+  (setq jde-project-name name))
+
 (defun jde-set-variables (&rest args)
   "Initialize JDE customization variables.  
 
@@ -993,21 +752,31 @@ The value of SYMBOL is set to VALUE.
       (if (listp entry)
 	  (let* ((symbol (nth 0 entry))
 		 (value (nth 1 entry))
-		 (now (nth 2 entry))
-		 (requests (nth 3 entry))
+		 (customized (nth 2 entry))
 		 (set (or (get symbol 'custom-set) 'set-default)))
-	    (put symbol 'saved-value (list value))
-	    (cond (now
-		   (put symbol 'force-value t)
-		   (funcall set symbol (eval value)))
-		  ((default-boundp symbol)
+	    (if customized
+		(put symbol 'customized-value (list value)))
+	    (when (default-boundp symbol)
 		   ;; Something already set this, overwrite it
-		   (funcall set symbol value)))
-	    (when requests
-	      (put symbol 'custom-requests requests)
-	      (mapcar 'require requests))
+		   (funcall set symbol (eval value)))
 	    (setq args (cdr args)))))))
 
+(defun jde-set-variables-init-value ()
+  "Set each JDE variable to the value it has at Emacs startup."
+  (interactive)
+  (message "Setting JDE variables to startup values...")
+  (mapatoms 
+   (lambda (symbol) 
+     (when 
+	 (and (string-match "jde-" (symbol-name symbol))
+	      (get symbol 'custom-type))
+       (let ((saved-val (get symbol 'saved-value))
+	     (std-val (get symbol 'standard-value))
+	     (set (or (get symbol 'custom-set) 'set-default)))
+	 (if saved-val
+	     (funcall set symbol (eval (car saved-val)))
+	   (funcall set symbol (eval (car std-val)))))))))
+ 
 ;;;###autoload
 (defun jde-save-project (proj-name)
   "Saves local source file buffer options in project file.
@@ -1030,6 +799,39 @@ tree, the saved settings will be restored for that file."
       (setq jde-project-name proj-name))
   (jde-save-variables))
 
+(defun jde-convert-prj-file (file) 
+"Converts a pre-JDE-2.0.7 project file to JDE-2.0.7 format.
+Note: old project files did not preserve information about 
+whether a saved value differed from the standard (JDE-defined)
+value of a variable. Thus, all values are saved in the
+converted file as though they were standard values. This means
+that when JDE reloads the file, a custom buffer will customized
+values as though they were standard. If you want to restore
+a customized value to a standard value, simply make some
+innocuous edit to the customized value and choose 
+'Set for current session' from the customization buffer's
+Set menu. Custom will then enable the Set menu option that
+allows you to restore the value to its default value."
+  (interactive "F")
+  (let ((olddef (symbol-function 'jde-set-variables))
+	(newdef 
+	 (lambda (&rest args)
+	   (while args 
+	     (let ((entry (car args)))
+	       (if (listp entry)
+		   (let* ((symbol (nth 0 entry))
+			  (value (nth 1 entry))
+			  (set (or (get symbol 'custom-set) 'set-default)))
+		     (when (default-boundp symbol)
+		       ;; Something already set this, overwrite it
+		       (funcall set symbol value))
+		     (setq args (cdr args)))))))))
+    (defalias 'jde-set-variables newdef)
+    (require 'cus-edit)
+    (load-file file)
+    (jde-save-project jde-project-name)
+    (defalias 'jde-set-variables olddef)))
+
 ;; Code to update JDE customization variables when a user switches
 ;; from a Java source buffer belonging to one project to a buffer
 ;; belonging to another.
@@ -1044,7 +846,13 @@ the new buffer's project differs from the old buffer's."
 	(setq jde-current-project jde-project-name)
 	(jde-load-project-file))))
 
-(add-hook 'jde-entering-java-buffer-hooks 'jde-reload-project-file)
+;; (add-hook 'jde-entering-java-buffer-hooks 'jde-reload-project-file)
+
+(defcustom jde-entering-java-buffer-hooks '(jde-reload-project-file)
+"*Lists functions to run when entering a Java source buffer"
+  :group 'jde-project
+  :type 'hook)
+
 
 (setq jde-current-buffer (current-buffer))
 
@@ -1084,30 +892,51 @@ all Java source buffers have been closed."
 
 ;; JDE help
 
-(defun jde-find-jde-directory ()
-  "Return the path of the JDE directory or nil if the
-directory cannot be found."
-  (if jde-xemacsp
-      (locate-data-directory "jde")
-    (find-if 
-     (lambda (dir) 
-       (file-exists-p 
-	(expand-file-name "jde.el" dir)))
-     load-path)))
+(defun jde-find-jde-data-directory ()
+  "Return the path of the JDE data directory.
+Returns  nil if the directory cannot be found. At some
+point, XEmacs will include the JDE. Versions of XEmacs
+that include JDE will store the JDE doc and Java clasees in a data
+directory called jde. On all other Emacs versions, the JDE
+expects to find the documentation and Java classes in subdirectories
+of the directory that contains the file jde.el."
+  (let ((dir (if jde-xemacsp
+		 (locate-data-directory "jde"))))
+    (if dir
+	dir
+      (file-name-directory (locate-library "jde")))))
 
+(defun jde-find-jde-doc-directory ()
+  "Return the path of the JDE documentation directory.
+Returns  nil if the directory cannot be found. At some
+point, XEmacs will include the JDE. Versions of XEmacs
+that include JDE will store the JDE doc in a data
+directory called jde. On all other Emacs versions, the JDE
+expects to find the documentation in a subdirectory 
+named doc of the directory that contains the file
+jde.el."
+  (jde-find-jde-data-directory))
+
+ 
 ;;;###autoload
 (defun jde-show-help ()
   "Displays the JDE User's Guide in a browser."
   (interactive)
-  (let* ((jde-dir (jde-find-jde-directory))
+  (let* ((jde-dir (jde-find-jde-doc-directory))
 	 (jde-help
-	  (if (and (not jde-xemacsp) jde-dir)
-	      (expand-file-name "doc/jde.htm" jde-dir)
-	    (expand-file-name "jde.htm" jde-dir))))
+	  (if jde-dir
+	      (if (and jde-xemacsp
+		       (locate-data-directory "jde"))
+		  (expand-file-name "jde-ug.html" jde-dir)
+		(expand-file-name "doc/jde-ug.html" jde-dir)))))	  
     (if jde-help
-	(browse-url (concat "file:" jde-help)  browse-url-new-window-p)
+	(if (and (eq system-type 'windows-nt)
+		 (not jde-xemacsp))
+	    (browse-url jde-help browse-url-new-window-p)
+	  (browse-url (concat "file:" jde-help)  browse-url-new-window-p))
       (signal 'error '("Cannot find JDE help file.")))))
 
+
 ;; speedbar
 
 (defun jde-make-imenu-patterns ()
@@ -1384,6 +1213,76 @@ pattern.
 ;; Change History
 ;;
 ;; $Log: jde.el $
+;; Revision 1.76  1998/12/09 00:56:31  paulk
+;; Put jde-compile variables in a new file jde-compile.el.
+;;
+;; Revision 1.75  1998/11/27 10:10:03  paulk
+;; Updated JDE version number to 2.1.3.
+;;
+;; Revision 1.74  1998/11/22 18:13:57  paulk
+;; Added menu items for the BeanShell and method override and interface wizards.
+;;
+;; Revision 1.73  1998/09/13 02:01:53  paulk
+;; Fixed a small bug in key binding code.
+;;
+;; Revision 1.72  1998/09/13 01:49:35  paulk
+;; Added support for customization of JDE key bindings via the
+;; variable jde-key-bindings.
+;;
+;; Revision 1.71  1998/09/13 00:32:48  paulk
+;; Added System.out.println template to the Generate menu.
+;;
+;; Revision 1.70  1998/09/07 02:50:31  paulk
+;; This version includes the latest version of jde-gen.el, which was inadvertently
+;; replaced by an older version in the last release. This version also includes
+;; a newer version of speedbar.el that seems to work better with NT/Emacs 20.3.1
+;; than the one that comes with the 20.3.1 distribution.
+;;
+;; Revision 1.69  1998/08/28 12:56:06  paulk
+;; *** empty log message ***
+;;
+;; Revision 1.68  1998/08/28 12:52:52  paulk
+;; Updated version number.
+;;
+;; Revision 1.67  1998/07/28 03:15:33  paulk
+;; Removed a diagnostic message.
+;;
+;; Revision 1.66  1998/07/28 03:12:40  paulk
+;; Updated version number to 2.0.9.
+;;
+;; Revision 1.65  1998/07/28 03:12:05  paulk
+;; Fixed the following project file bugs:
+;;
+;;   * JDE does not store the project name in the project file.
+;;   * JDE does not save variables whose value is nil.
+;;   * JDE does not reset variables to initial values when
+;;     switching to a buffer that is not part of a project.
+;;
+;; Revision 1.64  1998/07/22 00:10:07  paulk
+;; Now requires cus-edit. This fixes custom-quote is void bug.
+;;
+;; Fixed bug in jde-set-variables that prevented loading of
+;; project files in the new format.
+;;
+;; Revision 1.63  1998/07/10 00:49:24  paulk
+;; Changed jde-save-variables to mark variables that have been customized\n in the current session. Changed jde-set-variables to store the value\n of a customized variable in the customized-value property of the\n variable. This enables Custom to recognize the variable as customized.\n\n  Added jde-convert-prj-file, a function that converts old project files to \n \
+;; JDE-2.0.7 format.\n\n Fixed a bug in the function that finds the JDE documentation.
+;;
+;; Revision 1.62  1998/07/09 04:33:57  paulk
+;; Change the way that the JDE saves and restores project-specific values of
+;; customization variables to be compatible with custom. This fixes the bug
+;; that caused errors when loading customized JDE variables from a .emacs file.
+;;
+;; Revision 1.61  1998/07/04 05:25:15  paulk
+;; Should have been does not turn on font-lock if XEmacs 21.0.
+;;
+;; Revision 1.60  1998/07/04 00:53:58  paulk
+;; Now does not turn on font-lock if XEmacs 20.1.
+;;
+;; Revision 1.59  1998/07/02 05:33:13  paulk
+;; Fixed bugs in the jde-show-help function that prevented display
+;; of help on XEmacs and NT/Emacs.
+;;
 ;; Revision 1.58  1998/06/30 03:35:10  paulk
 ;; Added the customization variable `jde-read-compile-args'. If non-nil,
 ;; this variable causes the jde-compile command to read compilation options
